#include <Application.h>
#include <FilePanel.h>
#include <BitmapStream.h>
#include <TranslatorRoster.h>
#include <Bitmap.h>
#include <Message.h>
#include <Resources.h>
#include <stdlib.h>
#include <stdio.h>
#include <Mime.h>
#include <Node.h>
#include <NodeInfo.h>
#include <ResourceImage.h>

// Create and run the application object
main()
{
ResourceImageApp *resapp;
resapp = new ResourceImageApp("application/ResourceImage");
resapp->Run();
delete resapp;
}

// All we do is create and show a FilePanel

ResourceImageApp::ResourceImageApp(const char *signature) : BApplication(signature)
{
panel= new BFilePanel(B_OPEN_PANEL);
panel->SetTarget(this);
panel->Show();
}

ResourceImageApp::~ResourceImageApp()
{
delete panel; //delete the file panel
}

void ResourceImageApp::MessageReceived(BMessage *message)
{
	switch(message->what)
		{
		case B_CANCEL: //Cancel pressed on FilePanel so quit
			Quit();
			break;
		
		default:
			BApplication::MessageReceived(message);
			break;
		}
}


void ResourceImageApp::RefsReceived(BMessage *message)
{
	entry_ref 	ref;		// The entry_ref to open
	status_t 	err;		// The error code
	int32		ref_num;	// The index into the ref list
	
	// Ensure resource MIME type is installed
	
	BMimeType mime("application/x-be-resource");
	if (!mime.IsInstalled()) mime.Install();

	// Loop through the ref list and resource each one

	ref_num = 0;
	do {
		if ((err = message->FindRef("refs", ref_num, &ref)) != B_OK) {
			return;
		}
		CreateResourceFromRef(&ref, &mime);
		ref_num++;
	} while (1);
Quit();
}

//Very dirty no error checking :(

void ResourceImageApp::CreateResourceFromRef(entry_ref *ref, BMimeType *mime)
{
ssize_t size;
char *buffer;
char outfilename[255]; //should be long enough :)
char resname[255];
BResources res;
BBitmap *bitmap = NULL;

BFile infile(ref, B_READ_ONLY); //open selected file

BTranslatorRoster *roster = BTranslatorRoster::Default(); 
BBitmapStream stream; 
   if (roster->Translate(&infile, NULL, NULL, &stream, 
           B_TRANSLATOR_BITMAP) < B_OK) 
         return; //if not a valid bitmap (or no installed translator) abort conversion
stream.DetachBitmap(&bitmap); //create BBitmap from file

BMessage msg(B_ARCHIVED_OBJECT);
bitmap->Archive(&msg); // Archive BBitmap into BMessage
size=msg.FlattenedSize(); 
buffer = (char*)malloc(size);
msg.Flatten(buffer, size ); //Flatten (doh!)

sprintf(outfilename, "/boot/home/%s.rsrc",ref->name); //output in home directory
sprintf(resname, "%s_bitmap", ref->name); //label resource as mangled original file name

BFile outfile(outfilename, B_READ_WRITE | B_CREATE_FILE);

res.SetTo(&outfile, true); //clobber file if already exists
res.AddResource(B_RAW_TYPE,0,buffer,size, resname); //add resorce

BNode node(outfilename); 
BNodeInfo info(&node);
info.SetType(mime->Type()); //set the file's MIME type to "application/x-be-resource"

delete bitmap; //clean up
}


